 %Adaptive Slicing Script

close all;
clear all;
clc;

%Path Parameters

%Path Length, mm
PathLength = 3.147*25.4;

%Number of Beads
N_bead = 3;

%Bead Stepover, mm
Step = 3.29;

%Path Starts
X_start = 0;
X_stop = X_start + PathLength;
%Y_start = -1.1674;
Y_start = 0;

%Create Y toolpath
Y = zeros(1,N_bead);
for i = 1:N_bead
    Y(i) = Y_start + (i-1)*Step;
end

%Number of Probe Points
N = 10;

%Augmented Approximation, 1=yes, 0=no
aug = 1;
%Probe increment
Probe_Res = PathLength/(N-1);

%Initialize arrays for each bead
x = zeros(N, N_bead);
y = zeros(N, N_bead);

%Generate Probe Toolpath Coordinates
for i = 1:N_bead
    for j = 1:N
        %Formulate probe path for each bead
        x(j,i) = X_start + (j-1)*Probe_Res;
        y(j,i) = Y(i);
    end
end

%Read data from probe data files
[fileID, msg] = fopen('Probe_Data_10pts_SmallFront.txt','r');
if fileID < 0
   error('Failed to open file "%s" because: "%s"', 'Probe_Data_10pts_SmallFront.txt', msg);
end
d = textscan(fileID,'%f','delimiter','\n'); 
Z = d{1}*25.4;

%Transform matrices to vectors
x = x(:);
y = y(:);

%Make data matrix
Data = [x,y,Z];
%Sort data according to x value
[temp, order] = sort(Data(:,1));
Data = Data(order,:);

%Consolidate bead probe outputs
New_Data = zeros(N, 3);
j=1;
for i = 1:N
    %Parse out data segment
    Int_Data = [Data(j,:); Data(j+1,:); Data(j+2,:)];
    
    %Sort data segment according to Z value, in ascending order
    [temp, order] = sort(Int_Data(:,3));
    Int_Data = Int_Data(order,:);
    
    %Store row of Int_Data with highest Z value in New_Data
    New_Data(i,:) = Int_Data(3,:);
    
    j = j + 3;
end

figure
grid on
plot(New_Data(:,1), New_Data(:,3), '*-', 'LineWidth', 3)
xlabel('X (mm)')
ylabel('Z (mm)')
%title('Height Profile: Consolidated Probe Output')

%Absolute difference between highest and lowest probe point height values, mm
Absolute_Delta = max(New_Data(:,3))-min(New_Data(:,3));

%Difference between heights of path start and stop points, mm
Corner_Delta = abs(New_Data(1,3) - New_Data(size(New_Data,1),3));
%Expected Height Value, mm
Z_exp = 12;

% PathLength = 100;
% Absolute_Delta = 10;
% Corner_Delta = 2;

%%
%Create ground truth height profile
%Path Length, mm
PathLength = 3.147*25.4;

Probe_Res = PathLength/(10-1);
%Initialize arrays for each bead
x = zeros(10, N_bead);
y = zeros(10, N_bead);

%Generate Probe Toolpath Coordinates
for i = 1:N_bead
    for j = 1:10
        %Formulate probe path for each bead
        x(j,i) = X_start + (j-1)*Probe_Res;
        y(j,i) = Y(i);
    end
end

%Read data from probe data files
[fileID, msg] = fopen('Probe_Data_10pts_SmallFront.txt','r');
if fileID < 0
   error('Failed to open file "%s" because: "%s"', 'Probe_Data10pts_SmallFront.txt', msg);
end
d = textscan(fileID,'%f','delimiter','\n'); 
Z = d{1}*25.4;

%Transform matrices to vectors
x = x(:);
y = y(:);

%Make data matrix
GroundData = [x,y,Z];
%Sort data according to x value
[temp, order] = sort(GroundData(:,1));
GroundData = GroundData(order,:);

%Consolidate bead probe outputs
GroundNew_Data = zeros(10, 3);
j=1;
for i = 1:10
    %Parse out data segment
    Int_Data = [GroundData(j,:); GroundData(j+1,:); GroundData(j+2,:)];
    
    %Sort data segment according to Z value, in ascending order
    [temp, order] = sort(Int_Data(:,3));
    Int_Data = Int_Data(order,:);
    
    %Store row of Int_Data with highest Z value in New_Data
    GroundNew_Data(i,:) = Int_Data(3,:);
    
    j = j + 3;
end

%%
%Cup Geometry, Triangle Approximation
if N ==3
%Create height profile
H_Tri = [min(New_Data(:,3))+Absolute_Delta; min(New_Data(:,3)); min(New_Data(:,3))+Absolute_Delta-Corner_Delta];

%Create path coordinates
X_Tri = zeros(length(H_Tri),1);
for i = 1:length(H_Tri)
    X_Tri(i) = (i-1)*PathLength/(length(H_Tri)-1);
end

%Store height profile data
Tri_Profile = [X_Tri, H_Tri];

%Plot Height Profile
figure
plot(X_Tri,H_Tri,"*r-", 'LineWidth', 3);
hold on
plot(GroundNew_Data(:,1), GroundNew_Data(:,3), 'k*-', 'LineWidth', 3)
plot([0, 0, PathLength, PathLength],[H_Tri(1), 0, 0, H_Tri(3)],"*r-", 'LineWidth', 3);
plot([0, PathLength], [Z_exp, Z_exp], "*b--", 'LineWidth', 3)
%title("Triangular Approximation");
ylim([Z_exp max(H_Tri)])
xlabel("X (mm)")
ylabel("Z (mm)")

end
%%
%Cup Geometry, Trapezoidal Approximation
if N == 5
H_Trap = [min(New_Data(:,3))+Absolute_Delta; max([New_Data(2,3),New_Data(3,3),New_Data(4,3)]); max([New_Data(2,3),New_Data(3,3),New_Data(4,3)]); max([New_Data(2,3),New_Data(3,3),New_Data(4,3)]); min(New_Data(:,3))+Absolute_Delta-Corner_Delta];

%Create path coordinates
X_Trap = zeros(length(H_Trap),1);
for i = 1:length(H_Trap)
    X_Trap(i) = (i-1)*PathLength/(length(H_Trap)-1);
end

%Store height profile data
Trap_Profile = [X_Trap, H_Trap];

%Plot Height Profile
figure
plot(X_Trap,H_Trap,"*r-", 'LineWidth', 3);
hold on
plot(GroundNew_Data(:,1), GroundNew_Data(:,3), 'k*-', 'LineWidth', 3)
plot([0, 0, PathLength, PathLength],[H_Trap(1), 0, 0, H_Trap(5)],"*r-", 'LineWidth', 3);
plot([0, PathLength], [Z_exp, Z_exp], "*b--", 'LineWidth', 3)
%title("Trapezoidal Approximation");
ylim([Z_exp max(H_Trap)])
xlabel("X (mm)")
ylabel("Z (mm)")
end
%%
%Cup Geometry, Augmented Approximation
if N == 10

Aug_Data = [New_Data(1,1), New_Data(1,2), New_Data(1,3);
            New_Data(2,1), New_Data(2,2), New_Data(2,3);
            0.5*(New_Data(5,1)+New_Data(6,1)), 0.5*(New_Data(5,2)+New_Data(6,2)), 0.5*(New_Data(5,3)+New_Data(6,3));
            New_Data(9,1), New_Data(9,2), New_Data(9,3);
            New_Data(10,1), New_Data(10,2), New_Data(10,3)];
        
%Absolute difference between highest and lowest probe point height values, mm
Absolute_Delta = max(Aug_Data(:,3))-min(Aug_Data(:,3));

%Difference between heights of path start and stop points, mm
Corner_Delta = abs(Aug_Data(1,3) - Aug_Data(size(Aug_Data,1),3));

% H_Aug = [min(New_Data(:,3))+Absolute_Delta; max([New_Data(2,3),New_Data(3,3),New_Data(4,3)]); max([New_Data(2,3),New_Data(3,3),New_Data(4,3)]); max([New_Data(2,3),New_Data(3,3),New_Data(4,3)]); min(New_Data(:,3))+Absolute_Delta-Corner_Delta];
% 
% %Create path coordinates
% X_Aug = zeros(length(H_Aug),1);
% for i = 1:length(H_Aug)
%     X_Aug(i) = (i-1)*PathLength/(length(H_Aug)-1);
% end

%Store height profile data
X_Aug = Aug_Data(:,1);
H_Aug = Aug_Data(:,3);
Aug_Profile = [X_Aug, H_Aug];

%Calculate slope data
Aug_Slope_Data = [];

%Iterate through pairs of probe points to find slope of straight line
%connecting them
for count = 1:size(Aug_Profile,1)-1
    
    %Calculate slope and intercept of straight line between ith and i+1th point
    Coefficients = polyfit([Aug_Profile(count,1), Aug_Profile(count+1,1)], [Aug_Profile(count,2), Aug_Profile(count+1,2)], 1);
    
    %Store in Slope Data
    Aug_Slope_Data = cat(1, Aug_Slope_Data, [Aug_Profile(count,1),Aug_Profile(count,2), Aug_Profile(count+1,1), Aug_Profile(count+1,2), Coefficients(1), Coefficients(2)]);
end

%Plot Height Profile
figure
plot(X_Aug,H_Aug,"*r-", 'LineWidth', 3);
hold on
plot(New_Data(:,1), New_Data(:,3), 'k*-', 'LineWidth', 3)
plot([0, 0, PathLength, PathLength],[H_Aug(1), 0, 0, H_Aug(5)],"*r-", 'LineWidth', 3);
plot([0, PathLength], [Z_exp, Z_exp], "*b--", 'LineWidth', 3)
%title("Augmented Approximation");
ylim([Z_exp max(H_Aug)])
xlabel("X (mm)")
ylabel("Z (mm)")
end
%%
%Run optimization
%Material Data
%Brinell Hardness
BHN = 135;
%Surface Speed, mm/min. Converted from 225 SFM
Surf_Speed = 225*12*25.4;

%Tool Data
%Diameter, mm
D = 25.4;
%Radius, mm
r_mm = D/2;
%RPM
RPM = Surf_Speed/(pi*D);
%Number of teeth
Nt = 4;

%Cut Data for Constant Step Down
%Maximum allowable step down depth, mm
Step_Down = 0.5;
%Cut Feed, mm/min
%F_cut = 17.2 ipm;
F_cut_OG = 436;
%Non optimized force
Cut_nonopt = Step_Down*F_cut_OG;
%Feed per tooth, mm/t
sz_OG = F_cut_OG/(4*(RPM*Nt));
%Retract height, above each successive layer, mm
Retract = 5;

%Rapid Feed, mm/min. Given as maximum possible rapid traverse rate for
%Mazak VC-500A-5X
F_rapid = 25000;

%Initialize blank array for storing results
Results = [];

%Calculate Non-Optmized Machining Time
%Number of levels
N_Levels = ceil(Absolute_Delta/Step_Down);

Non_Optimized_Time = N_Levels*((r_mm/F_cut_OG) + (PathLength/F_cut_OG) +((2*Retract+PathLength+r_mm)/F_rapid));

%Initialize arrays for storing adaptive layer and machining time data
Adaptive_Layer = 0.01:0.01:Step_Down;
F_cut_new = 1:5:8000;

T_Tri_Total = [];
Cut_Force_Tri = [];
T_Trap_Total = [];
Cut_Force_Trap = [];
T_Aug_Total = [];
Cut_Force_Aug = [];
Check = [];

Surface_Roughness = [];
EntryPoints_Tri = [];
EntryPoints_Trap = [];

Cut_Parameters = [];
count = 0;
%Iterate through adaptive layer range of 0 to Step_Down
for i = 1:length(Adaptive_Layer)
    
    %Iterate through feedrates
    for f = 1:length(F_cut_new)
        
        %if Adaptive_Layer(i)*F_cut_new(f) >= Step_Down*F_cut_OG
            %Calculate new cutting feed rate to correspond to adaptive layer
            %thickness
            %F_cut_new = (Step_Down/Adaptive_Layer(i))*sz_OG*RPM*Nt;
    
            %If new cutting feedrate exceeds max feedrate of Mazak VCU-500A/5X,
            %which is 8000 mm/min, set new cutting feed rate to that value
            %if F_cut_new >= 8000
        
            %    F_cut_new = 8000;
        
            %end
    
            %Calculate new feed/tooth
            sz = F_cut_new(f)/(RPM*Nt);
    
    
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %Triangle Geometry Machining Time
            %Triangle Geometry slopes and intercepts
            m1_tri = -Absolute_Delta/(PathLength/2);
            b1_tri = Absolute_Delta;
            m2_tri = (Absolute_Delta - Corner_Delta)/(PathLength/2);
            b2_tri = Absolute_Delta - Corner_Delta - m2_tri*PathLength;
    
            %Stage 1
            %Number of stage 1 levels
            N_L1_tri = ceil(Corner_Delta/Step_Down);
    
            %Calculate optimized T1 section
            %Initialize optimized T1 time as 0
            opt_time_t1 = 0;
            for j = 1:N_L1_tri
        
                Z1_n = Absolute_Delta - j*Step_Down;
        
                opt_time_t1 = opt_time_t1 + ((Z1_n - b1_tri)/(m1_tri*F_cut_OG)) + ((PathLength - ((Z1_n - b1_tri)/m1_tri))/F_rapid);
        
            end
    
            T1_tri = N_L1_tri*((r_mm/F_cut_OG)+((2*Retract+PathLength+r_mm)/F_rapid)) + opt_time_t1;
    
            %Stage 2
            %Calculate stage 2 end (stage 3 start)
            Z_s3 = (r_mm*m1_tri*m2_tri + b2_tri*m1_tri - b1_tri*m2_tri)/(m1_tri - m2_tri);
    
            %Number of stage 2 levels
            N_L2_tri = ceil((Absolute_Delta - Corner_Delta - Z_s3)/Adaptive_Layer(i));
    
            %Calculate optimized T2 section
            %Initialize optimized T2 time as 0
            opt_time_t2 = 0;
            for j = 1:N_L2_tri
        
                Z1_n = Absolute_Delta - Corner_Delta - j*Adaptive_Layer(i);
                Z2_n = Absolute_Delta - Corner_Delta - j*Adaptive_Layer(i);
        
                opt_time_t2 = opt_time_t2 + ((Z1_n - b1_tri)/(m1_tri*F_cut_new(f))) + ((((Z2_n - b2_tri)/m2_tri) - ((Z1_n - b1_tri)/m1_tri))/F_rapid) + ((PathLength - ((Z2_n - b2_tri)/m2_tri))/F_cut_new(f));
        
            end
    
            T2_tri = N_L2_tri*((r_mm/F_cut_new(f))+((2*Retract+PathLength+r_mm)/F_rapid)) + opt_time_t2;
    
            %Stage 3
            N_L3_tri = floor((Z_s3/Adaptive_Layer(i)));
    
            T3_tri = N_L3_tri*(((r_mm + PathLength)/F_cut_new(f))+((2*Retract+PathLength+r_mm)/F_rapid));
    
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %Trapezoidal Machining Time
            %Trapezoidal Geometry slopes and intercepts
            m1_trap = -Absolute_Delta/(PathLength/4);
            b1_trap = Absolute_Delta;
            m4_trap = (Absolute_Delta - Corner_Delta)/(PathLength/4);
            b4_trap = Absolute_Delta - Corner_Delta - m4_trap*PathLength;
    
            %Stage 1
            %Number of stage 1 levels
            N_L1_trap = ceil(Corner_Delta/Step_Down);
    
            %Calculate optimized T1 section
            %Initialize optimized T1 time as 0
            opt_time_t1 = 0;
            for j = 1:N_L1_trap
        
                Z1_n = Absolute_Delta - j*Step_Down;
        
                opt_time_t1 = opt_time_t1 + ((Z1_n - b1_trap)/(m1_trap*F_cut_OG)) + ((PathLength - ((Z1_n - b1_trap)/m1_trap))/F_rapid);
        
            end
    
            T1_trap = N_L1_trap*((r_mm/F_cut_OG)+((2*Retract+PathLength+r_mm)/F_rapid)) + opt_time_t1;
    
            %Stage 2
            %Number of stage 2 levels
            N_L2_trap = floor((Absolute_Delta - Corner_Delta)/Adaptive_Layer(i));
    
            %Calculate optimized T2 section
            %Initialize optimized T2 time as 0
            opt_time_t2 = 0;
            for j = 1:N_L2_trap
        
                Z1_n = Absolute_Delta - Corner_Delta - j*Adaptive_Layer(i);
                Z4_n = Absolute_Delta - Corner_Delta - j*Adaptive_Layer(i);
        
                opt_time_t2 = opt_time_t2 + ((Z1_n - b1_trap)/(m1_trap*F_cut_new(f))) + ((((Z4_n - b4_trap)/m4_trap) - ((Z1_n - b1_trap)/m1_trap))/F_rapid) + ((PathLength - ((Z4_n - b4_trap)/m4_trap))/F_cut_new(f));
        
            end
    
            T2_trap = N_L2_trap*((r_mm/F_cut_new(f))+((2*Retract+PathLength+r_mm)/F_rapid)) + opt_time_t2;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            if N == 10
            %Augmented Approximation
            m1_aug = Aug_Slope_Data(1,5);
            b1_aug = Aug_Slope_Data(1,6);
            m2_aug = Aug_Slope_Data(2,5);
            b2_aug = Aug_Slope_Data(2,6);
            m3_aug = Aug_Slope_Data(3,5);
            b3_aug = Aug_Slope_Data(3,6);
            m4_aug = Aug_Slope_Data(4,5);
            b4_aug = Aug_Slope_Data(4,6);

            %Stage 1
            %Number of stage 1 levels
            N_L1_aug = ceil(Corner_Delta/Step_Down);
    
            %Calculate optimized T1 section
            %Initialize optimized T1 time as 0
            opt_time_t1 = 0;
            for j = 1:N_L1_aug
        
                Z1_n = Aug_Slope_Data(1,2) - j*Step_Down;
        
                opt_time_t1 = opt_time_t1 + ((Z1_n - b1_aug)/(m1_aug*F_cut_OG)) + ((PathLength - ((Z1_n - b1_aug)/m1_aug))/F_rapid);
        
            end
    
            T1_aug = N_L1_aug*((r_mm/F_cut_OG)+((2*Retract+PathLength+r_mm)/F_rapid)) + opt_time_t1;
            
            %Stage 2
            %Calculate stage 2 end (stage 3 start)
            Z_a3 = (r_mm*m2_aug*m3_aug + b3_aug*m2_aug - b2_aug*m3_aug)/(m2_aug - m3_aug);
    
            %Number of stage 2 levels
            N_L2_aug = ceil((Aug_Slope_Data(4,4) - Z_a3)/Adaptive_Layer(i));
    
            %Calculate optimized T2 section
            %Initialize optimized T2 time as 0
            opt_time_t2 = 0;
            for j = 1:N_L2_aug
                
                %Calculate Z level
                Z =  Aug_Slope_Data(4,4) - j*Adaptive_Layer(i);
                
                %Determine if level is connected lines 2 and 4
                if Z > Aug_Slope_Data(4,2) && Z <= Aug_Slope_Data(4,4)
                    Z2_n = Z;
                    Z4_n = Z;
        
                    opt_time_t2 = opt_time_t2 + ((((Z2_n - b2_aug)/m2_aug)+Aug_Slope_Data(1,2))*(1/F_cut_new(f))) + ((((Z4_n - b4_aug)/m4_aug) - ((Z2_n - b2_aug)/m2_aug))/F_rapid) + ((PathLength - ((Z4_n - b4_aug)/m4_aug))/F_cut_new(f));
                
                %Determine if level is connected lines 2 and 3
                elseif Z >= Z_a3 && Z <= Aug_Slope_Data(3,4)
                    Z2_n = Z;
                    Z3_n = Z;
        
                    opt_time_t2 = opt_time_t2 + ((((Z2_n - b2_aug)/m2_aug)+Aug_Slope_Data(1,2))*(1/F_cut_new(f))) + ((((Z3_n - b3_aug)/m3_aug) - ((Z2_n - b2_aug)/m2_aug))/F_rapid) + ((PathLength - ((Z3_n - b3_aug)/m3_aug))/F_cut_new(f));
                    
                end
            end
    
            T2_aug = N_L2_aug*((r_mm/F_cut_new(f))+((2*Retract+PathLength+r_mm)/F_rapid)) + opt_time_t2;
            
            %Stage 3
            N_L3_aug = floor(((Z_a3-Aug_Slope_Data(2,4))/Adaptive_Layer(i)));
    
            T3_aug = N_L3_aug*(((r_mm + PathLength)/F_cut_new(f))+((2*Retract+PathLength+r_mm)/F_rapid));
            
            %T_Aug_Total = cat(1, T_Aug_Total, (T1_aug + T2_aug + T3_aug));
            %Cut_Force_Aug = cat(1, Cut_Force_Aug, Adaptive_Layer(i)*F_cut_new(f));
            
            Check = cat(1,Check, [(T1_aug + T2_aug + T3_aug), Adaptive_Layer(i), F_cut_new(f)]);
            
            end
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %Determine if machining times are less than non-optimized
            if (N == 5 && ((T1_trap + T2_trap) <= Non_Optimized_Time))
                
                %Determine if optimized cutting force is less than
                %optimized
                if Adaptive_Layer(i)*F_cut_new(f) <= Step_Down*F_cut_OG
                
                    %Save trapezoidal results
                    T_Trap_Total = cat(1, T_Trap_Total, (T1_trap + T2_trap));
    
                    %Calculate average axial depth of cut over length of tool engagement
                    Cut_Force_Trap = cat(1, Cut_Force_Trap, Adaptive_Layer(i)*F_cut_new(f));
            
                    %Calculate number of entry points for stage 2
                    EntryPoints_Trap = cat(1, EntryPoints_Trap, 2*(N_L2_trap));
                    
                    %Calculate new surface roughness and store
                    Surface_Roughness = cat(1,Surface_Roughness, (sz/4));
            
                    %Save parameters
                    Cut_Parameters = cat(1, Cut_Parameters, [Adaptive_Layer(i), F_cut_new(f)]);
                end
            elseif (N == 3 && ((T1_tri + T2_tri + T3_tri) <= Non_Optimized_Time))
                %Determine if optimized cutting force is less than
                %optimized
                if Adaptive_Layer(i)*F_cut_new(f) <= Step_Down*F_cut_OG
                    %Save triangular results
                    T_Tri_Total = cat(1, T_Tri_Total, (T1_tri + T2_tri + T3_tri));
    
                    %Calculate cut force
                    Cut_Force_Tri = cat(1,Cut_Force_Tri, Adaptive_Layer(i)*F_cut_new(f));
            
                    %Calculate number of entry points for stage 2 and 3
                    EntryPoints_Tri = cat(1, EntryPoints_Tri, 2*(N_L3_tri + N_L2_tri));
                    
                    %Calculate new surface roughness and store
                    Surface_Roughness = cat(1,Surface_Roughness, (sz/4));
            
                    %Save parameters
                    Cut_Parameters = cat(1, Cut_Parameters, [Adaptive_Layer(i), F_cut_new(f)]);
                end
                
            elseif (N==10 && ((T1_aug + T2_aug + T3_aug) <= Non_Optimized_Time))
                %Determine if optimized cutting force is less than
                %optimized
                %Adaptive_Layer(i)*F_cut_new(f)
                if Adaptive_Layer(i)*F_cut_new(f) <= Step_Down*F_cut_OG
            
                    %Save triangular results
                    T_Aug_Total = cat(1, T_Aug_Total, (T1_aug + T2_aug + T3_aug));
    
                    %Calculate cut force
                    Cut_Force_Aug = cat(1,Cut_Force_Aug, Adaptive_Layer(i)*F_cut_new(f));
                    
                    %Calculate new surface roughness and store
                    Surface_Roughness = cat(1,Surface_Roughness, (sz/4));
            
                    %Save parameters
                    Cut_Parameters = cat(1, Cut_Parameters, [Adaptive_Layer(i), F_cut_new(f)]);
                end
            end
        %end
   end
end

Check2 = [];
for i = 1:size(Check,1)
    
    if Check(i,1) <= Non_Optimized_Time
        
        Check2 = cat(1, Check2, [Check(i,1),Check(i,2), Check(i,3)]);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Plot non-normalized Results
% 
% if N == 3
% figure
% plot3(Cut_Parameters(:,1), Cut_Parameters(:,2), T_Tri_Total, "-r")
% xlabel("Adaptive Layer Thickness (mm)")
% ylabel("New Feedrate (mm/min)")
% zlabel("Total Triangular Machining Time (min)")
% title("Triangular Geometry Optimization")
% 
% figure
% plot3(Cut_Parameters(:,1), Cut_Parameters(:,2), Cut_Force_Tri, "-b")
% xlabel("Adaptive Layer Thickness (mm)")
% ylabel("New Feedrate (mm/min)")
% zlabel("Cut Force")
% title("Triangular Geometry Optimization")
% 
% elseif N == 5
% figure
% plot3(Cut_Parameters(:,1), Cut_Parameters(:,2), T_Trap_Total, "-r")
% xlabel("Adaptive Layer Thickness (mm)")
% ylabel("New Feedrate (mm/min)")
% zlabel("Total Trapezoidal Machining Time (min)")
% title("Trapezoidal Geometry Optimization")
% 
% figure
% plot3(Cut_Parameters(:,1), Cut_Parameters(:,2), Cut_Force_Trap, "-b")
% xlabel("Adaptive Layer Thickness (mm)")
% ylabel("New Feedrate (mm/min)")
% zlabel("Cut Force")
% title("Trapezoidal Geometry Optimization")
% end
% 
% figure
% plot3(Cut_Parameters(:,1), Cut_Parameters(:,2), Surface_Roughness, "-r")
% xlabel("Adaptive Layer Thickness (mm)")
% ylabel("Surface_Roughness (mm)")
% title("Surface Roughness Range")
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Normalize results
%Initialize arrays for normalized data
Normalized_Cut_Parameters = zeros(size(Cut_Parameters,1),2);
Normalized_Tri_Time = zeros(size(Cut_Parameters,1),1);
Normalized_Cut_Force_Tri = zeros(size(Cut_Parameters,1),1);
Normalized_Trap_Time = zeros(size(Cut_Parameters,1),1);
Normalized_Cut_Force_Trap = zeros(size(Cut_Parameters,1),1);
Normalized_Aug_Time = zeros(size(Cut_Parameters,1),1);
Normalized_Cut_Force_Aug = zeros(size(Cut_Parameters,1),1);

Normalized_Surface_Roughness = zeros(size(Cut_Parameters,1),1);
Normalized_Entry_Points_Tri = zeros(size(Cut_Parameters,1),1);
Normalized_Entry_Points_Trap = zeros(size(Cut_Parameters,1),1);

ObjectiveFunction_Tri = zeros(size(Cut_Parameters,1),1);
ObjectiveFunction_Trap = zeros(size(Cut_Parameters,1),1);
ObjectiveFunction_Aug = zeros(size(Cut_Parameters,1),1);

%Objective function weights
w_t = 0.1;
w_r = 0.1;
w_f = 0.8;
w_e = 0;

%Normalize data with axes maximums
for i = 1:size(Cut_Parameters,1)
    
    %Filter out results that have longer machining times than non-optimized
    %if T_Tri_Total(i) <= Non_Optimized_Time || T_Trap_Total(i) <= Non_Optimized_Time
        %Cut Parameters = [Adaptive Layer Height, Adaptive Feed Rate]
        Normalized_Cut_Parameters(i,1) = (Cut_Parameters(i,1)/max(Cut_Parameters(:,1)));
        Normalized_Cut_Parameters(i,2) = (Cut_Parameters(i,2)/max(Cut_Parameters(:,2)));
    
        %Machining times
        %Normalized_Tri_Time(i) = T_Tri_Total(i)/max(T_Tri_Total);
        %Normalized_Trap_Time(i) = T_Trap_Total(i)/max(T_Trap_Total);
        %Normalized_Aug_Time(i) = T_Aug_Total(i)/max(T_Aug_Total);
        
        Normalized_Tri_Time(i) = T_Tri_Total(i)/Non_Optimized_Time;
        %Normalized_Trap_Time(i) = T_Trap_Total(i)/Non_Optimized_Time;
        %Normalized_Aug_Time(i) = T_Aug_Total(i)/Non_Optimized_Time;
        
        %Cutting forces
        %Normalized_Cut_Force_Tri(i) = (Cut_Force_Tri(i)/max(Cut_Force_Tri));
        %Normalized_Cut_Force_Trap(i) = (Cut_Force_Trap(i)/max(Cut_Force_Trap));
        %Normalized_Cut_Force_Aug(i) = (Cut_Force_Aug(i)/max(Cut_Force_Aug));
        
        Normalized_Cut_Force_Tri(i) = 1-(Cut_Force_Tri(i)/Cut_nonopt);
        %Normalized_Cut_Force_Trap(i) = 1-(Cut_Force_Trap(i)/Cut_nonopt);
        %Normalized_Cut_Force_Aug(i) = 1-(Cut_Force_Aug(i)/Cut_nonopt);
        
        %Entry points
        %Normalized_Entry_Points_Tri(i) = EntryPoints_Tri(i)/max(EntryPoints_Tri);
        %Normalized_Entry_Points_Trap(i) = EntryPoints_Trap(i)/max(EntryPoints_Trap);
    
        %Surface Roughness
        %Normalized_Surface_Roughness(i) = Surface_Roughness(i)/max(Surface_Roughness);
        Normalized_Surface_Roughness(i) = Surface_Roughness(i)/sz_OG;
    
        %Calculate objective function
        ObjectiveFunction_Tri(i) = w_t*Normalized_Tri_Time(i) + w_r*Normalized_Surface_Roughness(i) + w_f*Normalized_Cut_Force_Tri(i);
        %ObjectiveFunction_Trap(i) = w_t*Normalized_Trap_Time(i) + w_r*Normalized_Surface_Roughness(i) + w_f*Normalized_Cut_Force_Trap(i);
        %ObjectiveFunction_Aug(i) = w_t*Normalized_Aug_Time(i) + w_r*Normalized_Surface_Roughness(i) + w_f*Normalized_Cut_Force_Aug(i);
        
    %end
end

%Plot objective function
if N == 3
figure
pointsize = 30;
scatter(Normalized_Cut_Parameters(:,1), Normalized_Cut_Parameters(:,2), pointsize, ObjectiveFunction_Tri, "filled")
colorbar
hold on
Min_Value = 5;
for count = 1:size(Normalized_Cut_Parameters,1)
    
    %Save as test value for minimum
    if ObjectiveFunction_Tri(count) < Min_Value
        Min_Value = ObjectiveFunction_Tri(count);
        Min_Index = count;
    end
end
plot(Normalized_Cut_Parameters(Min_Index,1),Normalized_Cut_Parameters(Min_Index,2), '*b')
xlabel("Normalized Adaptive Layer Thickness")
ylabel("Normalized Adaptive Feed Rate")
ylabel(colorbar, "Objective Function Value")
title("Normalized Triangular Geometry Optimization")

%Compile Results
TriTimeImprovement = 100*(T_Tri_Total(Min_Index)-Non_Optimized_Time)/Non_Optimized_Time;
TriSurfaceImprovement = 100*(Surface_Roughness(Min_Index)-sz_OG)/sz_OG;
TriForceImprovement = 100*(Cut_Force_Tri(Min_Index)-Cut_nonopt)/Cut_nonopt;
TriangularResults = [T_Tri_Total(Min_Index), Cut_Force_Tri(Min_Index), EntryPoints_Tri(Min_Index), Cut_Parameters(Min_Index,1), Cut_Parameters(Min_Index,2), Surface_Roughness(Min_Index), ObjectiveFunction_Tri(Min_Index)];
TriangularImprovement = [TriTimeImprovement, TriSurfaceImprovement, TriForceImprovement];
    
elseif N == 5
figure
pointsize = 30;
scatter(Normalized_Cut_Parameters(:,1), Normalized_Cut_Parameters(:,2), pointsize, ObjectiveFunction_Trap, "filled")
colorbar
hold on
Min_Value = 5;
for count = 1:size(Normalized_Cut_Parameters,1)
    
    %Save as test value for minimum
    if ObjectiveFunction_Trap(count) < Min_Value
        Min_Value = ObjectiveFunction_Trap(count);
        Min_Index = count;
    end
end
plot(Normalized_Cut_Parameters(Min_Index,1),Normalized_Cut_Parameters(Min_Index,2), '*b')
xlabel("Normalized Adaptive Layer Thickness")
ylabel("Normalized Adaptive Feed Rate")
ylabel(colorbar, "Objective Function Value")
title("Normalized Trapezoidal Geometry Optimization")

%Compile Results
TrapTimeImprovement = 100*(T_Trap_Total(Min_Index)-Non_Optimized_Time)/Non_Optimized_Time;
TrapSurfaceImprovement = 100*(Surface_Roughness(Min_Index)-sz_OG)/sz_OG;
TrapForceImprovement = 100*(Cut_Force_Trap(Min_Index)-Cut_nonopt)/Cut_nonopt;
TrapezoidalResults = [T_Trap_Total(Min_Index), Cut_Force_Trap(Min_Index), EntryPoints_Trap(Min_Index), Cut_Parameters(Min_Index,1), Cut_Parameters(Min_Index,2), Surface_Roughness(Min_Index), ObjectiveFunction_Trap(Min_Index)];
TrapezoidalImprovement = [TrapTimeImprovement, TrapSurfaceImprovement, TrapForceImprovement];

elseif N == 10
figure
pointsize = 30;
scatter(Normalized_Cut_Parameters(:,1), Normalized_Cut_Parameters(:,2), pointsize, ObjectiveFunction_Aug, "filled")
colorbar
hold on
Min_Value = 5;
for count = 1:size(Normalized_Cut_Parameters,1)
    
    %Save as test value for minimum
    if ObjectiveFunction_Aug(count) < Min_Value
        Min_Value = ObjectiveFunction_Aug(count);
        Min_Index = count;
    end
end
plot(Normalized_Cut_Parameters(Min_Index,1),Normalized_Cut_Parameters(Min_Index,2), '*b')
xlabel("Normalized Adaptive Layer Thickness")
ylabel("Normalized Adaptive Feed Rate")
ylabel(colorbar, "Objective Function Value")
title("Normalized Augmented Geometry Optimization")

%Compile Results
AugTimeImprovement = 100*(T_Aug_Total(Min_Index)-Non_Optimized_Time)/Non_Optimized_Time;
AugSurfaceImprovement = 100*(Surface_Roughness(Min_Index)-sz_OG)/sz_OG;
AugForceImprovement = 100*(Cut_Force_Aug(Min_Index)-Cut_nonopt)/Cut_nonopt;
AugmentedResults = [T_Aug_Total(Min_Index), Cut_Force_Aug(Min_Index), Cut_Parameters(Min_Index,1), Cut_Parameters(Min_Index,2), Surface_Roughness(Min_Index), ObjectiveFunction_Aug(Min_Index)];
AugmentedImprovement = [AugTimeImprovement, AugSurfaceImprovement, AugForceImprovement];
end
%%
%Adaptively Slice Height Profile
%Select triangle or trapezoid profile for processing
if N == 3
    Profile = Tri_Profile;
elseif N == 5
    Profile = Trap_Profile;
elseif N==10
    Profile = Aug_Profile;
end

%Find equations of straight lines between successive probe points
%Initialize array for storing probe points and the slopes of the lines
%connecting them
Slope_Data = [];

%Iterate through pairs of probe points to find slope of straight line
%connecting them
for i = 1:size(Profile,1)-1
    
    %Calculate slope and intercept of straight line between ith and i+1th point
    Coefficients = polyfit([Profile(i,1), Profile(i+1,1)], [Profile(i,2), Profile(i+1,2)], 1);
    
    %Store in Slope Data
    Slope_Data = cat(1, Slope_Data, [Profile(i,1),Profile(i,2), Profile(i+1,1), Profile(i+1,2), Coefficients(1), Coefficients(2)]);
end

%Tool/Cut Data
%Diameter, mm
D = 25;
%Radius, mm
r_mm = D/2;
%Maximum allowable step down depth, mm
Step_Down = 0.5;

%Initialize empty array of points for optimized toolpath. Array will
%contain start and stop points of toolpath segment [X1, X2, Z1, Z2] as well as whether
%the toolpath segment is cutting (1) or rapid (2)
OptimizedPoints = [];

%Start Plot of Optimized Toolpath
figure
plot(Profile(:,1),Profile(:,2),"*r-", 'LineWidth', 2);
hold on
plot([0, PathLength], [Z_exp, Z_exp], "*b--")

if N == 3
    plot([0, 0, PathLength, PathLength],[H_Tri(1), 0, 0, H_Tri(3)],"*r-", 'LineWidth', 2);
elseif N == 5
    plot([0, 0, PathLength, PathLength],[H_Trap(1), 0, 0, H_Trap(5)],"*r-", 'LineWidth', 2);
elseif N == 10
    plot([0, 0, PathLength, PathLength],[H_Aug(1), 0, 0, H_Aug(5)],"*r-", 'LineWidth', 2);
end

%Stage 1: Machine part from high corner Z level to low
%corner Z level. Use max step down

%Determine high and low corner Z levels
High_Corner = max(Profile(1,2), Profile(size(Profile,1),2));
Low_Corner = min(Profile(1,2), Profile(size(Profile,1),2));

%Number of Stage 1 Levels
N_Levels_1 = ceil((High_Corner-Low_Corner)/Step_Down);

%Plot Line to show stage 1 end height
plot([Profile(1,1)-r_mm, Profile(size(Profile,1),1)], [Low_Corner, Low_Corner], "m--", 'LineWidth',3)


for i = 1:N_Levels_1
    
    %Z Level value
    Level = High_Corner - i*Step_Down;
    if Level > Low_Corner
    %Determine what straight line equation is being used for this level 
    for j = 1:size(Slope_Data,1)
        
        %Check if Z level value is between Z values of start/stop points
        %for each line
        if min([Slope_Data(j,2) Slope_Data(j,4)]) < Level && Level < max([Slope_Data(j,2) Slope_Data(j,4)])
            
            %Calculate intersection X value
            Intercept = Slope_Data(j,6);
            Slope = Slope_Data(j,5);
            
            X_Int = (Level - Intercept)/Slope;
        end
    end
    
    %Plot cut to perimeter of part
    plot([Profile(1,1)-r_mm, Profile(1,1)], [Level, Level], "*b-", 'LineWidth',1)
    
    %Store Optimized Toolpath Points 
    OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1)-r_mm, Profile(1,1), Level, Level, 1]);
    
    %Plot cut to intersection of straight line between probe points
    plot([Profile(1,1), X_Int ], [Level, Level], "*b-", 'LineWidth',1)
    
    %Store Optimized Toolpath Points 
    OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1), X_Int, Level, Level, 1]);
    
    %Plot rapid move out 
    plot([X_Int, Profile(size(Profile,1),1)],[Level, Level], "*g-", 'LineWidth',1);
    
    %Store Optimized Toolpath Points 
    OptimizedPoints =  cat(1, OptimizedPoints, [X_Int, Profile(size(Profile,1),1), Level, Level, 2]);
    
    end
end

%Stage 2. Machine from end of stage 1 height until local maxima using
%adaptive slicing to keep axial tool engagement near constant. 

%For triangle geometries, instead of local maxima, set stage 2 lowest Z level to be the
%Z level where the distance between either side is one tool radius. 
if size(Profile,1) == 3
    
    %Determine Z level such that distance between lines of triangle is
    %equal to one tool radius and store as stage 2 end height
    Stage2_End = (r_mm*Slope_Data(1,5)*Slope_Data(2,5) + Slope_Data(1,5)*Slope_Data(2,6) - Slope_Data(2,5)*Slope_Data(1,6))/(Slope_Data(1,5)-Slope_Data(2,5));
    
    %Plot line to represent stage 2 end height
    plot([Profile(1,1)-r_mm, Profile(size(Profile,1),1)], [Stage2_End, Stage2_End], "c--", 'LineWidth',3)
    
    %Find adaptive slice thickness 
    %Calculate Z layer height based on allowable increase in X
    Z_allowed = TriangularResults(2);
    %Z_allowed = 0.1;
   
    %Set Z_allowed to max step down if Z_allowed exceeds max step down
    if Z_allowed > Step_Down
        Z_allowed = Step_Down;
    end
    
    %Find number of levels for stage 2
    N_Levels_2 = floor((Low_Corner-Stage2_End)/Z_allowed);
    
    %Plot optimized toolpath for stage 2
    for i = 1:N_Levels_2
    
        %Z Level value
        Level = Low_Corner - i*Z_allowed;
        
        if Level > Stage2_End
            %Calculate intersection X value for first line
            Intercept1 = Slope_Data(1,6);
            Slope1 = Slope_Data(1,5);
            X_Int1 = (Level - Intercept1)/Slope1;
    
            %Calculate intersection X value for second line
            Intercept2 = Slope_Data(2,6);
            Slope2 = Slope_Data(2,5);
            X_Int2 = (Level - Intercept2)/Slope2;
    
            %Plot cut to perimeter of part
            plot([Profile(1,1)-r_mm, Profile(1,1)], [Level, Level], "*b-", 'LineWidth',1)
   
            %Store Optimized Toolpath Points 
            OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1)-r_mm, Profile(1,1), Level, Level, 1]);
    
            %Plot cut to intersection of straight line between probe points
            plot([Profile(1,1), X_Int1 ], [Level, Level], "*b-", 'LineWidth',1)

            %Store Optimized Toolpath Points 
            OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1), X_Int1, Level, Level, 1]);
            
            %Plot rapid move to next intersection 
            plot([X_Int1, X_Int2-r_mm],[Level, Level], "*g-", 'LineWidth',1);
            
            %Store Optimized Toolpath Points 
            OptimizedPoints =  cat(1, OptimizedPoints, [X_Int1, X_Int2-r_mm, Level, Level, 2]);
    
            %Plot cut move to end of part
            plot([X_Int2-r_mm, Profile(size(Profile,1),1)],[Level, Level], "*b-", 'LineWidth',1);
            
            %Store Optimized Toolpath Points 
            OptimizedPoints =  cat(1, OptimizedPoints, [X_Int2-r_mm, Profile(size(Profile,1),1), Level, Level, 1]);

        end
    end
end


%For trapezoidal geometries, set stage 2 bottom as "base" of trapezoid
%(highest height of two middle points)
if N == 5
    
    %Determine Z level such that distance between lines of triangle is
    %equal to one tool radius and store as stage 2 end height
    Stage2_End = max([Profile(2,2), Profile(3,2), Profile(4,2)]);
    
    
    %Plot line to represent stage 2 end height
    plot([Profile(1,1)-r_mm, Profile(size(Profile,1),1)], [Stage2_End, Stage2_End], "c--", 'LineWidth',3)
    
    %Find adaptive slice thickness 
    %Calculate Z layer height based on allowable increase in X
    Z_allowed = Cut_Parameters(Min_Index,1);
    %Z_allowed = 0.1;
    %Set Z_allowed to max step down if Z_allowed exceeds max step down
    if Z_allowed > Step_Down
        Z_allowed = Step_Down;
    end
    
    %Find number of levels for stage 3
    N_Levels_2 = floor((Low_Corner-Stage2_End)/Z_allowed);
    
    %Plot optimized toolpath for stage 2
    for i = 1:N_Levels_2
    
        %Z Level value
        Level = Low_Corner - i*Z_allowed;
        
        if Level > Stage2_End
            %Calculate intersection X value for first line
            Intercept1 = Slope_Data(1,6);
            Slope1 = Slope_Data(1,5);
            X_Int1 = (Level - Intercept1)/Slope1;
    
            %Calculate intersection X value for second line
            Intercept2 = Slope_Data(4,6);
            Slope2 = Slope_Data(4,5);
            X_Int2 = (Level - Intercept2)/Slope2;
    
            %Plot cut to perimeter of part
            plot([Profile(1,1)-r_mm, Profile(1,1)], [Level, Level], "*b-", 'LineWidth',1)

            %Store Optimized Toolpath Points 
            OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1)-r_mm, Profile(1,1), Level, Level, 1]);
            
            %Plot cut to intersection of straight line between probe points
            plot([Profile(1,1), X_Int1 ], [Level, Level], "*b-", 'LineWidth',1)

            %Store Optimized Toolpath Points 
            OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1), X_Int1, Level, Level, 1]);
            
            %Plot rapid move to next intersection 
            plot([X_Int1, X_Int2-r_mm],[Level, Level], "*g-", 'LineWidth',1);

            %Store Optimized Toolpath Points 
            OptimizedPoints =  cat(1, OptimizedPoints, [X_Int1, X_Int2-r_mm, Level, Level, 2]);
            
            %Plot cut move to end of part
            plot([X_Int2-r_mm, Profile(size(Profile,1),1)],[Level, Level], "*b-", 'LineWidth',1);
            
            %Store Optimized Toolpath Points 
            OptimizedPoints =  cat(1, OptimizedPoints, [X_Int2-r_mm, Profile(size(Profile,1),1), Level, Level, 1]);
        end
    end
    
    End_Height_Trap = Level;
end


%Stage 3: Machine from stage 2 end to triangle low point. 

%For triangle geometries:
if size(Profile,1) == 3
    
    %Continue to machine down to lowest height profile point using adaptive
    %slice thickness
    
    %Set Stage 3 start level as last level of Stage 2 minus one adaptive
    %slice
    Stage3_Start = Level;
    
    %Calculate number of stage 3 levels
    N_Levels_3 = floor((Stage3_Start-min(Profile(:,2)))/Z_allowed);
    
    %Plot stage 3 end point
    plot([Profile(1,1)-r_mm, Profile(size(Profile,1),1)], [min(Profile(:,2)), min(Profile(:,2))], "r--", 'LineWidth',3)
    
    %Plot optimized toolpath for stage 3
    for i = 1:N_Levels_3
        
        %Calculate Level value
        Level = Stage3_Start - i*Z_allowed;
    
        %Plot cut to perimeter of part
        plot([Profile(1,1)-r_mm, Profile(1,1)], [Level, Level], "*b-", 'LineWidth',1)

        %Store Optimized Toolpath Points 
        OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1)-r_mm, Profile(1,1), Level, Level, 1]);
            
        %Plot cut to end of part
        plot([Profile(1,1), Profile(size(Profile,1),1) ], [Level, Level], "*b-", 'LineWidth',1)

        %Store Optimized Toolpath Points 
        OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1), Profile(size(Profile,1),1), Level, Level, 1]);
    end
    
    End_Height_Tri = Level;
end

%For augmented geometries, instead of local maxima, set stage 2 lowest Z level to be the
%Z level where the distance between either side is one tool radius. 
if N == 10
    
    %Determine Z level such that distance between lines of triangle is
    %equal to one tool radius and store as stage 2 end height
    Stage2_End = (r_mm*Slope_Data(2,5)*Slope_Data(3,5) + Slope_Data(2,5)*Slope_Data(3,6) - Slope_Data(3,5)*Slope_Data(2,6))/(Slope_Data(2,5)-Slope_Data(3,5));
    
    %Plot line to represent stage 2 end height
    plot([Profile(1,1)-r_mm, Profile(size(Profile,1),1)], [Stage2_End, Stage2_End], "c--", 'LineWidth',3)
    
    %Find adaptive slice thickness 
    %Calculate Z layer height based on allowable increase in X
    Z_allowed = AugmentedResults(3);
    %Z_allowed = 0.1;
    %Set Z_allowed to max step down if Z_allowed exceeds max step down
    if Z_allowed > Step_Down
        Z_allowed = Step_Down;
    end
    
    %Find number of levels for stage 2
    N_Levels_2 = floor((Low_Corner-Stage2_End)/Z_allowed);
    
    %Plot optimized toolpath for stage 2
    for i = 1:N_Levels_2
    
        %Z Level value
        Level = Low_Corner - i*Z_allowed;
        
        if Level > Stage2_End
            if Level > Slope_Data(4,2) && Level < Slope_Data(4,4)
                %Calculate intersection X value for second line
                Intercept2 = Slope_Data(2,6);
                Slope2 = Slope_Data(2,5);
                X_Int2 = ((Level - Intercept2)/Slope2);
    
                %Calculate intersection X value for fourth line
                Intercept4 = Slope_Data(4,6);
                Slope4 = Slope_Data(4,5);
                X_Int4 = ((Level - Intercept4)/Slope4);
    
                %Plot cut to perimeter of part
                plot([Profile(1,1)-r_mm, Profile(1,1)], [Level, Level], "*b-", 'LineWidth',1)
   
                %Store Optimized Toolpath Points 
                OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1)-r_mm, Profile(1,1), Level, Level, 1]);
    
                %Plot cut to intersection of straight line between probe points
                plot([Profile(1,1), X_Int2 ], [Level, Level], "*b-", 'LineWidth',1)

                %Store Optimized Toolpath Points 
                OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1), X_Int2, Level, Level, 1]);
            
                %Plot rapid move to next intersection 
                plot([X_Int2, X_Int4-r_mm],[Level, Level], "*g-", 'LineWidth',1);
            
                %Store Optimized Toolpath Points 
                OptimizedPoints =  cat(1, OptimizedPoints, [X_Int2, X_Int4-r_mm, Level, Level, 2]);
    
                %Plot cut move to end of part
                plot([X_Int4-r_mm, Profile(size(Profile,1),1)],[Level, Level], "*b-", 'LineWidth',1);
            
                %Store Optimized Toolpath Points 
                OptimizedPoints =  cat(1, OptimizedPoints, [X_Int4-r_mm, Profile(size(Profile,1),1), Level, Level, 1]);
            
            elseif Level > Slope_Data(3,2) && Level < Slope_Data(3,4)
                %Calculate intersection X value for second line
                Intercept2 = Slope_Data(2,6);
                Slope2 = Slope_Data(2,5);
                X_Int2 = ((Level - Intercept2)/Slope2);
    
                %Calculate intersection X value for third line
                Intercept3 = Slope_Data(3,6);
                Slope3 = Slope_Data(3,5);
                X_Int3 = ((Level - Intercept3)/Slope3);
    
                %Plot cut to perimeter of part
                plot([Profile(1,1)-r_mm, Profile(1,1)], [Level, Level], "*b-", 'LineWidth',1)
   
                %Store Optimized Toolpath Points 
                OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1)-r_mm, Profile(1,1), Level, Level, 1]);
    
                %Plot cut to intersection of straight line between probe points
                plot([Profile(1,1), X_Int2 ], [Level, Level], "*b-", 'LineWidth',1)

                %Store Optimized Toolpath Points 
                OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1), X_Int2, Level, Level, 1]);
            
                %Plot rapid move to next intersection 
                plot([X_Int2, X_Int3-r_mm],[Level, Level], "*g-", 'LineWidth',1);
            
                %Store Optimized Toolpath Points 
                OptimizedPoints =  cat(1, OptimizedPoints, [X_Int2, X_Int3-r_mm, Level, Level, 2]);
    
                %Plot cut move to end of part
                plot([X_Int3-r_mm, Profile(size(Profile,1),1)],[Level, Level], "*b-", 'LineWidth',1);
            
                %Store Optimized Toolpath Points 
                OptimizedPoints =  cat(1, OptimizedPoints, [X_Int3-r_mm, Profile(size(Profile,1),1), Level, Level, 1]);
                
            end
        end
    end
    
    %Continue to machine down to lowest height profile point using adaptive
    %slice thickness
    
    %Set Stage 3 start level as last level of Stage 2 minus one adaptive
    %slice
    Stage3_Start = Level;
    
    %Calculate number of stage 3 levels
    N_Levels_3 = ceil((Stage3_Start-min(Profile(:,2)))/Z_allowed);
    
    %Plot stage 3 end point
    plot([Profile(1,1)-r_mm, Profile(size(Profile,1),1)], [min(Profile(:,2)), min(Profile(:,2))], "r--", 'LineWidth',3)
    
    %Plot optimized toolpath for stage 3
    for i = 1:N_Levels_3
        
        %Calculate Level value
        Level = Stage3_Start - i*Z_allowed;
    
        %Plot cut to perimeter of part
        plot([Profile(1,1)-r_mm, Profile(1,1)], [Level, Level], "*b-", 'LineWidth',1)

        %Store Optimized Toolpath Points 
        OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1)-r_mm, Profile(1,1), Level, Level, 1]);
            
        %Plot cut to end of part
        plot([Profile(1,1), Profile(size(Profile,1),1) ], [Level, Level], "*b-", 'LineWidth',1)

        %Store Optimized Toolpath Points 
        OptimizedPoints =  cat(1, OptimizedPoints, [Profile(1,1), Profile(size(Profile,1),1), Level, Level, 1]);
    end
    
    End_Height_Tri = Level;
end


title("Optimized Toolpath");
ylim([Z_exp max(Profile(:,2))])
xlabel("X (mm)")
ylabel("Z (mm)")


%%
%Graph 10pt plot surface and find slope data
%Path Length, mm
PathLength = 3.147*25.4;

%Number of Beads
N_bead = 3;

%Bead Stepover, mm
Step = 3.29;

%Path Starts
X_start = 0;
X_stop = X_start + PathLength;
%Y_start = -1.1674;
Y_start = 0;

%Create Y toolpath
Y = zeros(1,N_bead);
for i = 1:N_bead
    Y(i) = Y_start + (i-1)*Step;
end

%Number of Probe Points
N = 10;
%Probe increment
Probe_Res = PathLength/(N-1);

%Initialize arrays for each bead
x = zeros(N, N_bead);
y = zeros(N, N_bead);

%Generate Probe Toolpath Coordinates
for i = 1:N_bead
    for j = 1:N
        %Formulate probe path for each bead
        x(j,i) = X_start + (j-1)*Probe_Res;
        y(j,i) = Y(i);
    end
end

%Read data from probe data files
[fileID, msg] = fopen('Probe_Data_10pts_SmallFront.txt','r');
if fileID < 0
   error('Failed to open file "%s" because: "%s"', 'Probe_Data_10pts_SmallFront.txt', msg);
end
d = textscan(fileID,'%f','delimiter','\n'); 
Z = d{1}*25.4;

%Transform matrices to vectors
x = x(:);
y = y(:);

%Make data matrix
Data = [x,y,Z];
%Sort data according to x value
[temp, order] = sort(Data(:,1));
Data = Data(order,:);

%Consolidate bead probe outputs
New_Data = zeros(N, 3);
j=1;
for i = 1:N
    %Parse out data segment
    Int_Data = [Data(j,:); Data(j+1,:); Data(j+2,:)];
    
    %Sort data segment according to Z value, in ascending order
    [temp, order] = sort(Int_Data(:,3));
    Int_Data = Int_Data(order,:);
    
    %Store row of Int_Data with highest Z value in New_Data
    New_Data(i,:) = Int_Data(3,:);
    
    j = j + 3;
end

%Find equations of straight lines between successive probe points
%Initialize array for storing probe points and the slopes of the lines
%connecting them
Slope_Data = [];

%Iterate through pairs of probe points to find slope of straight line
%connecting them
for i = 1:size(New_Data,1)-1
    
    %Calculate slope and intercept of straight line between ith and i+1th point
    Coefficients = polyfit([New_Data(i,1), New_Data(i+1,1)], [New_Data(i,3), New_Data(i+1,3)], 1);
    
    %Store in Slope Data
    Slope_Data = cat(1, Slope_Data, [New_Data(i,1), New_Data(i+1,1), New_Data(i,3), New_Data(i+1,3), Coefficients(1), Coefficients(2)]);
end

%%
%Overlay optimized toolpath onto 10pt profile and determine graph of axial
%load vs cutting time
figure
grid on

%Plot consolidated probe data
plot(New_Data(:,1), New_Data(:,3), '*r-', 'LineWidth', 2)
hold on

%Add lines to complete geometry
plot([0, 0, PathLength, PathLength],[New_Data(1,3), 0, 0, New_Data(size(New_Data,1),3)],"*r-", 'LineWidth', 2)

%Add expected Z line
plot([0, PathLength], [Z_exp, Z_exp], "*m--", 'LineWidth', 3)

%Initialize array for storing plot of axial depth of cut verus cut length
%in X. Store as (Xstart, Xstop, AverageA_DoC)
Axial_X = [];

%Divide optimized points matrix into submatrices based on Z value
[~,~,X] = unique(OptimizedPoints(:,3));
C = accumarray(X,1:size(OptimizedPoints,1),[],@(r){OptimizedPoints(r,:)});
%Add optimized toolpath
int = 1;
for i = 1:size(C,1)
    
    c = size(C,1) - (i-1);
    
    %Sort C by Xstart value
    [~,idx] = sort(C{c}(:,1)); 
    C{c} = C{c}(idx,:);
    
    for k = 1:size(C{c},1)
        
        if C{c}(k,5) == 1
        
            %Plot cut segment
            plot([C{c}(k,1), C{c}(k,2)], [C{c}(k,3), C{c}(k,4)], "*b-", 'LineWidth',1)
            
            %Filter out segments that are offset by one radius
            if abs(C{c}(k,1)) ~= r_mm
                
                %Initialize array for points of interest as (X, Z)
                Opt_Points_Interest = [];
                
                %Set level value
                Level = C{c}(k,3);
              
            
                %Determine intercept points
                for p = 1:size(Slope_Data,1)
                
                    %Determine if line segment is in level
                    if min(Slope_Data(p,3),Slope_Data(p,4)) < C{c}(k,3) && C{c}(k,3) < max(Slope_Data(p,3),Slope_Data(p,4))
                        
                        %Determine if line segment is in X bound of cut
                        %segment
                        if (C{c}(k,1) <= Slope_Data(p,1) && Slope_Data(p,1) <= C{c}(k,2)) || (C{c}(k,1) <= Slope_Data(p,2) && Slope_Data(p,2) <= C{c}(k,2))
                            
                            %Find intersection point
                            Intercept = Slope_Data(p,6);
                            Slope = Slope_Data(p,5);
                            X_Int = (C{c}(k,3) - Intercept)/Slope;
                    
                            plot(X_Int,C{c}(k,3), "ok")
                    
                            Opt_Points_Interest = cat(1, Opt_Points_Interest, [X_Int, Level]);
                            
                                if Level < Slope_Data(p,4) && Slope_Data(p,4) < Prev_Height
                
                                    Opt_Points_Interest = cat(1, Opt_Points_Interest, [Slope_Data(p,2), Slope_Data(p,4)]);
                    
                                    plot(Slope_Data(p,2), Slope_Data(p,4), "xk")
                                end
                        end
                    end
                
                end
                
                %Check if toolpath segment starts at path start and ends
                %somewhere that is not the path end
                if C{c}(k,1) == 0 && C{c}(k,2) ~= PathLength
                    
                    %Set Previous Height for very first segment, previous height is high point of
                    %profile
                    if c == size(C,1)
                        Prev_Height = Slope_Data(1,3);
                    
                        %Add first point to Opt_Points-Interest
                        Opt_Points_Interest = cat(1,Opt_Points_Interest, [Slope_Data(1,1), Slope_Data(1,3)]);
                        plot(Slope_Data(1,1), Slope_Data(1,3), "xk")
                    else
                    
                        %Add first point's X-coordinate with this Level's Z
                        %coordinate
                        Opt_Points_Interest = cat(1,Opt_Points_Interest, [Slope_Data(1,1), Level]);
                    end
                    
                    %Sort NonOpt_Points_Interest
                    [~,idx] = sort(Opt_Points_Interest(:,1)); 
                    Opt_Points_Interest = Opt_Points_Interest(idx,:);
                    
                    %Calculate aver axial depth of cut over toolpath engagement
                    Axial_X = cat(1, Axial_X, [Opt_Points_Interest(1,1), Opt_Points_Interest(2,1), Prev_Height - Level]);
                
                %Check if toolpath segment starts somewhere that is not the
                %path start and ends at PathLength
                elseif C{c}(k,1) ~= 0 && C{c}(k,2) == PathLength
        
                    %Add last point to Opt_Points_Interest
                    Opt_Points_Interest = cat(1,Opt_Points_Interest, [PathLength, Slope_Data(size(Slope_Data,1),4)]);
                    plot(PathLength, Slope_Data(size(Slope_Data,1),4), "xk")
                
                    %Sort NonOpt_Points_Interest
                    [~,idx] = sort(Opt_Points_Interest(:,1)); 
                    Opt_Points_Interest = Opt_Points_Interest(idx,:);
                
                
                    %Check if this optimized toolpath segment is the last
                    %cut segment of that level (ie this toolpath segment
                    %ends at the PathLength)
                    if Opt_Points_Interest(size(Opt_Points_Interest,1),1) == PathLength
                        
                        %Check if last point of profile is in between
                        %Prev_Height and level
                        if Level < Opt_Points_Interest(size(Opt_Points_Interest,1),2) && Opt_Points_Interest(size(Opt_Points_Interest,1),2) < Prev_Height
                            
                            %Calculate aver axial depth of cut over toolpath engagement
                            Axial_X = cat(1, Axial_X, [Opt_Points_Interest(1,1), Opt_Points_Interest(size(Opt_Points_Interest,1),1), Opt_Points_Interest(size(Opt_Points_Interest,1),2) - Level]);
                        else
                            
                            %Calculate aver axial depth of cut over toolpath engagement
                            Axial_X = cat(1, Axial_X, [Opt_Points_Interest(1,1), Opt_Points_Interest(size(Opt_Points_Interest,1),1), Prev_Height - Level]);
                        end
                    end
                
                %Check if toolpath segment starts somewhere that isn't the
                %path start and ends somewhere that isn't the path end
                elseif C{c}(k,1) ~= 0 && C{c}(k,2) ~= PathLength
                
                %Check if toolpath segment starts at the path start and
                %ends at the path end
                elseif C{c}(k,1) == 0 && C{c}(k,2) == PathLength
                    
                    %Add first point to Opt_Points-Interest
                    Opt_Points_Interest = cat(1,Opt_Points_Interest, [Slope_Data(1,1), Level]);
                    
                    %Add last point to Opt_Points-Interest
                    Opt_Points_Interest = cat(1,Opt_Points_Interest, [PathLength, Level]);
                    
                    %Sort NonOpt_Points_Interest
                    [~,idx] = sort(Opt_Points_Interest(:,1)); 
                    Opt_Points_Interest = Opt_Points_Interest(idx,:);
                    
                    %Check if this toolpath segment has full air cutting between it's
                    %engagement with the two sides of the path profile (ie
                    %that it has 4 Opt_Points)
                    if size(Opt_Points_Interest,1) == 4
                        
                        %Calculate average axial depth of cut over toolpath engagement
                        Axial_X = cat(1, Axial_X, [Opt_Points_Interest(1,1), Opt_Points_Interest(2,1), Prev_Height - Level]);
                        Axial_X = cat(1, Axial_X, [Opt_Points_Interest(3,1), Opt_Points_Interest(4,1), Prev_Height - Level]);
                    
                    %Check if this toolpath segment has enagement with the
                    %part in between the engagement with the two sides of
                    %the path profile (ie it has more than 4 Opt_Points
                    elseif size(Opt_Points_Interest,1) > 4
                        
                        %Calculate average axial depth of cut over toolpath engagement
                        Axial_X = cat(1, Axial_X, [Opt_Points_Interest(1,1), Opt_Points_Interest(2,1), Prev_Height - Level]);
                        
                        for point = 3:size(Opt_Points_Interest,1)-2
                            
                            %Check if there are Opt_Points that are between
                            %Prev_Height and the current Level
                            if Level < Opt_Points_Interest(point,2) && Opt_Points_Interest(point,2) < Prev_Height
                                
                                %Calculate average axial depth of cut over length
                                %of tool engagement
                                Axial_X = cat(1, Axial_X, [Opt_Points_Interest(point-1,1), Opt_Points_Interest(point+1,1), Opt_Points_Interest(point,2) - Level]);
                            
                            %If all OptPoints are equal to Level value
                            elseif Opt_Points_Interest(point,2) == Level && Opt_Points_Interest(point+1,2) == Level
                                
                                %Check that only points between start and
                                %end engagement are used
                                if point+1 ~= size(Opt_Points_Interest,1)-1
                                    %Calculate average axial depth of cut over length
                                    %of tool engagement
                                    Axial_X = cat(1, Axial_X, [Opt_Points_Interest(point,1), Opt_Points_Interest(point+1,1), Prev_Height - Level]);
                                end
                                 
                            end
                        end
                        
                        %Calculate average axial depth of cut over toolpath engagement
                        Axial_X = cat(1, Axial_X, [Opt_Points_Interest(size(Opt_Points_Interest,1)-1,1), Opt_Points_Interest(size(Opt_Points_Interest,1),1), Prev_Height - Level]);

                    end

                end
                
            end

        elseif C{c}(k,5) == 2
        
            plot([C{c}(k,1), C{c}(k,2)], [C{c}(k,3), C{c}(k,4)], "*g-", 'LineWidth',1)
        
        end
    end
    %Set previous height
    Prev_Height = Level;
end

% %Add non-optimized portion of toolpath until Z expected is reached
% N_Levels_Opt_10 = floor((OptimizedPoints(size(OptimizedPoints,1),3)-Z_exp)/Step_Down);
% 
% for i = 1:N_Levels_Opt_10
%     
%     Level = OptimizedPoints(size(OptimizedPoints,1),3) - i*Step_Down;
%     
%     %Plot cut to perimeter of part
%     plot([New_Data(1,1)-r_mm, New_Data(1,1)], [Level, Level], "*b-", 'LineWidth',1)
%             
%     %Plot cut to end of part
%     plot([New_Data(1,1), New_Data(size(New_Data,1),1) ], [Level, Level], "*b-", 'LineWidth',1)
%     
%     %Determine axial load over cut segment
%         for k = 1:size(Slope_Data,1)
%               
%             if min(Slope_Data(k,3),Slope_Data(k,4)) < Level && Level < max(Slope_Data(k,3),Slope_Data(k,4))
%                     %Find intersection point
%                     Intercept = Slope_Data(k,6);
%                     Slope = Slope_Data(k,5);
%                     X_Int = (Level - Intercept)/Slope;
%                     
%                     %plot(X_Int, Level, "ok")
%             end
%         end
% end
% 
% %Plot last pass
% plot([New_Data(1,1)-r_mm, New_Data(1,1)], [Z_exp, Z_exp], "*b-", 'LineWidth',1)
% plot([New_Data(1,1), New_Data(size(New_Data,1),1) ], [Z_exp, Z_exp], "*b-", 'LineWidth',1)

xlabel('X (mm)')
ylabel('Z (mm)')
ylim([Z_exp max(New_Data(:,3))])
%title('10 pt Consolidated Probe Output With Optimized Toolpath')

TotalSamples = 0;
TotalSum = 0;
for i = 1:size(Axial_X,1)
    
    NumberSamples = 10000*(Axial_X(i,2) - Axial_X(i,1));
    
    TotalSum = TotalSum + Axial_X(i,3)*NumberSamples;
    TotalSamples = TotalSamples + NumberSamples;
end

%Average axial DoC
Average_Axial_Opt = TotalSum/TotalSamples;
%%
%Plot non-optimized toolpath over 10pt probe surface

figure
%Plot consolidated probe data
plot(New_Data(:,1), New_Data(:,3), '*r-', 'LineWidth', 2)
hold on

%Add lines to complete geometry
plot([0, 0, PathLength, PathLength],[New_Data(1,3), 0, 0, New_Data(size(New_Data,1),3)],"*r-", 'LineWidth', 2)

%Add expected Z line
plot([0, PathLength], [Z_exp, Z_exp], "*m--", 'LineWidth', 3)

%Check if triangular or trapezoidal and set end height accordingly
if size(Profile,1) == 3
    
    %Add non-optimized portion of toolpath until Z expected is reached
    N_Levels_10 = floor((max(New_Data(:,3))-End_Height_Tri)/Step_Down);
elseif size(Profile,1) == 5
    
    %Add non-optimized portion of toolpath until Z expected is reached
    N_Levels_10 = floor((max(New_Data(:,3))-End_Height_Trap)/Step_Down);    
end

%Initialize array for storing plot of axial depth of cut verus cut length
%in X. Store as (Xstart, Xstop, AverageA_DoC)
Axial_X = [];

for i = 1:N_Levels_10
    
    Level = max(New_Data(:,3)) - i*Step_Down;
    
    %Initialize array for points of interest as (X, Z)
    NonOpt_Points_Interest = [];
    
    if i == 1
        %Add first point to NonOpt_Points-Interest
        NonOpt_Points_Interest = cat(1,NonOpt_Points_Interest, [Slope_Data(1,1), Slope_Data(1,3)]);
        plot(Slope_Data(1,1), Slope_Data(1,3), "xk")
        
        %Add last point to NonOpt_Points_Interest
        NonOpt_Points_Interest = cat(1,NonOpt_Points_Interest, [PathLength, Slope_Data(size(Slope_Data,1),4)]);
        plot(PathLength, Slope_Data(size(Slope_Data,1),4), "xk")
        
    else
%         %Store lowest Z value point of interest
%         NonOpt_Points_Interest = cat(1,NonOpt_Points_Interest, LowestNonOpt);
        %Add first point to NonOpt_Points-Interest
        NonOpt_Points_Interest = cat(1,NonOpt_Points_Interest, [Slope_Data(1,1), Level]);
        plot(Slope_Data(1,1), Level, "xk")
        
        %Add last point to NonOpt_Points_Interest
        NonOpt_Points_Interest = cat(1,NonOpt_Points_Interest, [PathLength, Slope_Data(size(Slope_Data,1),4)]);
        plot(PathLength, Slope_Data(size(Slope_Data,1),4), "xk")
    end
    
    %Plot cut to perimeter of part
    plot([New_Data(1,1)-r_mm, New_Data(1,1)], [Level, Level], "*b-", 'LineWidth',1)
            
    %Plot cut to end of part
    plot([New_Data(1,1), New_Data(size(New_Data,1),1) ], [Level, Level], "*b-", 'LineWidth',1)
    
        %Determine points of interest
        for k = 1:size(Slope_Data,1)
              
            if min(Slope_Data(k,3),Slope_Data(k,4)) < Level && Level < max(Slope_Data(k,3),Slope_Data(k,4))
                    %Find intersection point
                    Intercept = Slope_Data(k,6);
                    Slope = Slope_Data(k,5);
                    X_Int = (Level - Intercept)/Slope;
                    
                    plot(X_Int, Level, "ok")
                    
                    NonOpt_Points_Interest = cat(1, NonOpt_Points_Interest, [X_Int, Level]);
            end
            
%             if Level < Slope_Data(k,4) && Slope_Data(k,4) < (Level+Step_Down)
%                 
%                     NonOpt_Points_Interest = cat(1, NonOpt_Points_Interest, [Slope_Data(k,2), Slope_Data(k,4)]);
%                     
%                     plot(Slope_Data(k,2), Slope_Data(k,4), "xk")
%             end
        end
        
        %Find all points of interest that lie on level line
%         LowestNonOpt = [];
%         for count = 1:size(NonOpt_Points_Interest,1)
%             if NonOpt_Points_Interest(count,2) == Level
%                 LowestNonOpt = cat(1,LowestNonOpt, [NonOpt_Points_Interest(count,1), NonOpt_Points_Interest(count,2)]);
%             end
%         end

        
        %Find average depths of cut over cut length
        if i == 1
                            %Sort NonOpt_Points_Interest
                [~,idx] = sort(NonOpt_Points_Interest(:,1)); 
                NonOpt_Points_Interest = NonOpt_Points_Interest(idx,:);
                
%             Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(1,1), NonOpt_Points_Interest(2,1), (NonOpt_Points_Interest(1,2)-NonOpt_Points_Interest(2,2))/2]);
            Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(1,1), NonOpt_Points_Interest(2,1), Step_Down]);
        else
            
            %Check if cutting is below surface (ie regular cutting)
%             if isempty(NonOpt_Points_Interest) == 1
%                 
%                 Axial_X = cat(1, Axial_X, [New_Data(1,1), New_Data(size(New_Data,1),1), Step_Down]);
%             else
                %Sort NonOpt_Points_Interest
                [~,idx] = sort(NonOpt_Points_Interest(:,1)); 
                NonOpt_Points_Interest = NonOpt_Points_Interest(idx,:);
%             
%                 Axial_X = cat(1, Axial_X, [New_Data(1,1), NonOpt_Points_Interest(1,1), Step_Down]);
            
            
                for n = 1: size(NonOpt_Points_Interest,1)-1
                      
                      %Check if NonOpt_Points are at the beginning of part
                      if NonOpt_Points_Interest(n,1) == 0
                          
                          Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(n,1), NonOpt_Points_Interest(n+1,1), Step_Down]);
                          
                      %Check if NonOpt_Points are at the end of the part
                      elseif NonOpt_Points_Interest(n+1,1) == PathLength
                          
                          if NonOpt_Points_Interest(n+1,2)-NonOpt_Points_Interest(n,2) > 0 && NonOpt_Points_Interest(n+1,2)-NonOpt_Points_Interest(n,2) < Step_Down
                            Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(n,1), NonOpt_Points_Interest(n+1,1), NonOpt_Points_Interest(n+1,2)-NonOpt_Points_Interest(n,2)]);
                          else
                            Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(n,1), NonOpt_Points_Interest(n+1,1), Step_Down]);
                          end
                      else
                          Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(n,1), NonOpt_Points_Interest(n+1,1), 0]);
                      end
                    
%                     %Check if NonOpt_Points Z value is on level line
%                     if NonOpt_Points_Interest(n+1,2) == Level
% % %                        Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(n,1), NonOpt_Points_Interest(n+1,1), abs((NonOpt_Points_Interest(n,2)-NonOpt_Points_Interest(n+1,2))/2)]);
%                          Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(n,1), NonOpt_Points_Interest(n+1,1), Step_Down]);
%                          
%                     elseif NonOpt_Points_Interest(n+1,2) < NonOpt_Points_Interest(n,2)
% %                         Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(n,1), NonOpt_Points_Interest(n+1,1), abs((NonOpt_Points_Interest(n,2)-NonOpt_Points_Interest(n+1,2))/2)+(NonOpt_Points_Interest(n+1,2)-Level)]);
%                         Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(n,1), NonOpt_Points_Interest(n+1,1), Step_Down]);
%                     elseif NonOpt_Points_Interest(n+1,2) > NonOpt_Points_Interest(n,2)
%                          Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(n,1), NonOpt_Points_Interest(n+1,1), abs((NonOpt_Points_Interest(n,2)-NonOpt_Points_Interest(n+1,2))/2)+(NonOpt_Points_Interest(n,2)-Level)]);
%                         %Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(n,1), NonOpt_Points_Interest(n+1,1), Step_Down]);
%                     end
                
                end
            
                %Axial_X = cat(1, Axial_X, [NonOpt_Points_Interest(size(NonOpt_Points_Interest,1),1), New_Data(size(New_Data,1),1), Step_Down]);
            
            %end
        end
end

xlabel('X (mm)')
ylabel('Z (mm)')
ylim([Z_exp max(New_Data(:,3))])
%title('10 pt Consolidated Probe Output With Non-Optimized Toolpath')

%Get rid of all Axial_X rows that have zero magntidue
New_Axial_X = [];
for j = 1:size(Axial_X,1)
    
    if Axial_X(j,3) ~= 0
        New_Axial_X = cat(1, New_Axial_X, Axial_X(j,:));
    end
end


%Determine cumulative length of cut engagement
%Initialize array of cut engagemant data as (Xstart, Xstop, Z)
Cut_Engagement = [];
Cut_Start = 0;
for i = 1: size(New_Axial_X,1)
    
    Magnitude = New_Axial_X(i,3);
    if i == 1
            
        Cut_Engagement = cat(1, Cut_Engagement, [Cut_Start, New_Axial_X(i,2), Magnitude]);
        
        Cut_Intermediate = New_Axial_X(i,2);
    else
            
        Cut_Engagement = cat(1, Cut_Engagement, [Cut_Intermediate, New_Axial_X(i,2)-New_Axial_X(i,1)+Cut_Intermediate, Magnitude]);
        
        Cut_Intermediate = New_Axial_X(i,2)-New_Axial_X(i,1)+Cut_Intermediate;
    end
end

TotalSamples = 0;
TotalSum = 0;
for i = 1:size(Cut_Engagement,1)
    
    NumberSamples = 10000*(Cut_Engagement(i,2) - Cut_Engagement(i,1));
    
    TotalSum = TotalSum + Cut_Engagement(i,3)*NumberSamples;
    TotalSamples = TotalSamples + NumberSamples;
end

%Average axial DoC
Average_Axial = TotalSum/TotalSamples;

%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %Plot normalized results
% %Initialize arrays for normalized data
% Normalized_Adaptive = zeros(length(Adaptive_Layer),1);
% Normalized_Tri_Time = zeros(length(Adaptive_Layer),1);
% 
% %Normalize data with axes maximums
% for i = 1:length(Adaptive_Layer)
%     
%     Normalized_Adaptive(i) = Adaptive_Layer(i)/max(Adaptive_Layer);
%     Normalized_Tri_Time(i) = T_Tri_Total(i)/max(T_Tri_Total); 
% end
% 
% figure
% plot(Normalized_Adaptive, Normalized_Tri_Time, "-r")
% hold on
% 
% %Calculate optimal points as shortest distance to origin (ideal point since
% %goal is to minimize total machining time and adaptive layer thickness)
% Optimal_Distance = zeros(length(Normalized_Adaptive),3);
% Min_Value = 5;
% for i = 1:length(Normalized_Adaptive)
%     Optimal_Distance(i,1) = Adaptive_Layer(i);
%     Optimal_Distance(i,2) = T_Tri_Total(i);
%     Optimal_Distance(i,3) = sqrt((Normalized_Adaptive(i)^2) + (Normalized_Tri_Time(i)^2));
%     
%     %Save as test value for minimum
%     if Optimal_Distance(i,3) < Min_Value
%         Min_Value = Optimal_Distance(i,3);
%         Min_Index = i;
%     end
% end
% 
% 
% %Plot Optimal point
% plot(Normalized_Adaptive(Min_Index), Normalized_Tri_Time(Min_Index), '*b')
% 
% xlabel("Normalized Adaptive Layer Thickness")
% ylabel("Normalized Total Triangular Machining Time")
% title("Normalized Triangular Geometry Optimization")
% 
% %Compile Results
% PercentDiff = 100*abs(T_Tri_Total(Min_Index)-Non_Optimized_Time)/(0.5*(T_Tri_Total(Min_Index)+Non_Optimized_Time));
% TriangularResults = [T_Tri_Total(Min_Index), Adaptive_Layer(Min_Index), PercentDiff];
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %Plot normalized results
% %Initialize arrays for normalized data
% Normalized_Adaptive = zeros(length(Adaptive_Layer),1);
% Normalized_Trap_Time = zeros(length(Adaptive_Layer),1);
% 
% %Normalize data with axes maximums
% for i = 1:length(Adaptive_Layer)
%     
%     Normalized_Adaptive(i) = Adaptive_Layer(i)/max(Adaptive_Layer);
%     Normalized_Trap_Time(i) = T_Trap_Total(i)/max(T_Trap_Total); 
% end
% 
% figure
% plot(Normalized_Adaptive, Normalized_Trap_Time, "-r")
% hold on
% 
% %Calculate optimal points as shortest distance to origin (ideal point since
% %goal is to minimize total machining time and adaptive layer thickness)
% Optimal_Distance = zeros(length(Normalized_Adaptive),3);
% Min_Value = 5;
% for i = 1:length(Normalized_Adaptive)
%     Optimal_Distance(i,1) = Adaptive_Layer(i);
%     Optimal_Distance(i,2) = T_Trap_Total(i);
%     Optimal_Distance(i,3) = sqrt((Normalized_Adaptive(i)^2) + (Normalized_Trap_Time(i)^2));
%     
%     %Save as test value for minimum
%     if Optimal_Distance(i,3) < Min_Value
%         Min_Value = Optimal_Distance(i,3);
%         Min_Index = i;
%     end
% end
% 
% 
% %Plot Optimal point
% plot(Normalized_Adaptive(Min_Index), Normalized_Trap_Time(Min_Index), '*b')
% 
% xlabel("Normalized Adaptive Layer Thickness")
% ylabel("Normalized Total Trapezoidal Machining Time")
% title("Normalized Trapezoidal Geometry Optimization")
% 
% %Compile Results
% PercentDiff = 100*abs(T_Trap_Total(Min_Index)-Non_Optimized_Time)/(0.5*(T_Trap_Total(Min_Index)+Non_Optimized_Time));
% TrapezoidalResults = [T_Trap_Total(Min_Index), Adaptive_Layer(Min_Index), PercentDiff];
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Store results
%Results =  cat(1, Results, [PathLength(a), Corner_Delta(b), Absolute_Delta(c), TriangularResults, TrapezoidalResults]);


%%
% %Plot Height Profile
% %Initialize arrays
% x_sim = zeros(2*N,1);
% y_sim = zeros(2*N,1);
% x_act = zeros(N,1);
% y_act = zeros(N,1);
% k = 1;
% 
% %Create Graph Points
% for i=1:length(H_probe)-1
%     
%     x_act(i) = X_off + (i-1)*Probe_Res;
%     y_act(i) = H_probe(i);
%     
%     %Ascending
%     if H_probe(i) < H_probe(i+1) 
%         x_sim(k) = X_off + (i-1)*Probe_Res;
%         x_sim(k+1) = X_off + (i-1)*Probe_Res;
%         y_sim(k) = H_probe(i);
%         y_sim(k+1) = H_probe(i+1);
%         
%         k = k + 2;
%     elseif H_probe(i) > H_probe(i+1) 
%         x_sim(k) = X_off + (i-1)*Probe_Res;
%         x_sim(k+1) = X_off + (i)*Probe_Res;
%         y_sim(k) = H_probe(i);
%         y_sim(k+1) = H_probe(i);
%         
%         k = k + 2;
%     elseif H_probe(i) == H_probe(i+1)
%         x_sim(k) = X_off + (i-1)*Probe_Res;
%         x_sim(k+1) = X_off + (i)*Probe_Res;
%         y_sim(k) = H_probe(i);
%         y_sim(k+1) = H_probe(i);
%         
%         k = k + 2;
%     end
% end
% 
% x_act(N) = X_off + (N-1)*Probe_Res;
% y_act(N) = H_probe(N);
%         
% x_sim(k) = x_act(N);
% y_sim(k) = y_act(N);
% %Plot Height Profile
% %Clear Zeros from plot data
% x_sim = x_sim(x_sim ~= 0);
% y_sim = y_sim(y_sim ~= 0);
% 
% figure
% plot(x_sim,y_sim,"*r-");
% hold on
% plot(x_act, y_act, "*b-");
% title("Actual vs. Simulated");
% xlabel("X (mm)")
% ylabel("Z (mm)")
% legend("Simulated", "Actual");

%    Xf = Profile(size(Profile,1),1);
%     %Quadratic formula coefficients
%     a = -Slope_Data(2,5);
%     b = Y1 - Slope_Data(2,6) + (Xf -X1)*Slope_Data(2,5) + Adaptive_Res*(Xf -X1)*Slope_Data(2,5);
%     c = (Xf -X1)*Slope_Data(2,6) - (Xf -X1)*Y1 + Adaptive_Res*Slope_Data(2,6)*(Xf -X1) - Adaptive_Res*(Xf -X1)*Y1;
%     
%     X2 = [(-b + sqrt((b^2)-4*a*c))/(2*a), (-b - sqrt((b^2)-4*a*c))/(2*a)];
%     Y2 = [Slope_Data(2,5)*(Xf - X2(1))+Slope_Data(2,6), Slope_Data(2,5)*(Xf - X2(2))+Slope_Data(2,6)];
%     
%     %Plot end point lines
%     plot([Xf - X2(1), Profile(size(Profile,1),1)], [Y2(1), Y2(1)], "+k--")
%     %plot([Xf - X2(2), Profile(size(Profile,1),1)], [Y2(2), Y2(2)], "dk--")